* Runs simulation to check approximation given in appendix
* 1. Use theoretical expectations in approximate expression
* 2. Substitute empirical values into approximate expression
* 3. Use empirical values, no approximation

capture log close bigsim
log using bigsim.smcl, s replace name(bigsim)

version 15
clear *
timer on 1

mata
	rseed(180471)
	ntimes = 1000
	nsim = 10000 // no. repetitionss
	m = 7 // no. studies

	ee = 10:/range(1,m,1)
	//ee = J(1,m,61256.25) // alternative version with all studies same
	ff = range(1,m,1)
	//ff = J(1,m,618.75) // alternative

	df = 20:+3:*(range(m,1,1))
	//df = J(1,m,99) // alternative
	g2 = 1:/(df:-2)
	g4 = 1:/(df:-4)
	g6 = 1:/(df:-6)
	g8 = 1:/(df:-8)

	covab = 0
	vara = 0
	ea = 0
	eb = 0
	eb2 = 0
	flag = 0
	dum = 0
	for (i=1; i<=m; i++) {
//		printf("i=%g \n",i)
		ea = ea + ee[i]*g2[i]*g4[i]
		vx = 8*(df[i]-5)*g2[i]*g2[i]*g4[i]*g4[i]*g6[i]*g8[i]
		vara = vara + ee[i]*ee[i]*vx
		covab = covab + ee[i]*ff[i]*ff[i]*vx
		eb = eb + (ff[i]^2)*g2[i]*g4[i]
		eb2 = eb2 + (ff[i]^4)*g2[i]*g4[i]*g6[i]*g8[i]
		dum = dum + (ff[i]^4)
		flag = flag + 1
		for (j=1; j<=m; j++) {
			nij = (i != j)
//			printf("  j=%g, nij=%g\n",j,nij)
			covx = 4*g2[i]*g2[i]*g4[i]*g6[i]*g2[j]
			covab = covab + 2*nij*ee[i]*ff[i]*ff[j]*covx
			eb = eb + nij*ff[i]*ff[j]*g2[i]*g2[j]
			eb2 = eb2 + 4*nij*(ff[i]^3)*ff[j]*g2[i]*g4[i]*g6[i]*g2[j]
			dum = dum + 4*nij*(ff[i]^3)*ff[j]
			eb2 = eb2 + 3*nij*(ff[i]^2)*(ff[j]^2)*g2[i]*g4[i]*g2[j]*g4[j]
			dum = dum + 3*nij*(ff[i]^2)*(ff[j]^2)
			flag = flag + 7*nij
			for (k=1; k<=m; k++) {
				nik = (i != k)
				njk = (j != k)
				eb2 = eb2 + 6*nij*njk*nik*(ff[i]^2)*ff[j]*ff[k]*g2[i]*g4[i]*g2[j]*g2[k]
				dum = dum + 6*nij*njk*nik*(ff[i]^2)*ff[j]*ff[k]
				flag = flag + 6*nij*njk*nik
				for (l=1; l<=m; l++) {
					nil = (i != l)
					njl = (j != l)
					nkl = (k != l)
					eb2 = eb2 + nij*nik*nil*njk*njl*nkl*ff[i]*ff[j]*ff[k]*ff[l]*g2[i]*g2[j]*g2[k]*g2[l]
					dum = dum + nij*nik*nil*njk*njl*nkl*ff[i]*ff[j]*ff[k]*ff[l]
					flag = flag+nij*nik*nil*njk*njl*nkl
				}
			}
		}
	}
	varb = eb2 - (eb^2)


/* Simulation */

	vvlist = J(0,4,.)
	obsealist = J(0,1,.)
	obseblist = J(0,1,.)
	obseclist = J(0,1,.)
	for (itime=1; itime<=ntimes; itime++) {
		res = J(nsim,3,0)
	  if (itime == 1) printf("{txt}Simulation running:\n  ")
		if (round(itime/10) != round((itime-1)/10)) printf("{res}.")
		for (isim=1; isim<=nsim; isim++) {
			num = 0
			den = 0
			z = J(m,1,0)
			for (i=1; i<=m; i++) {
				z[i] = 1/rchi2(1,1,df[i])
				num = num + ee[i]*z[i]*z[i]
				den = den + ff[i]*z[i]
			} // closes loop over m
			den = den^2
			res[isim,1] = num
			res[isim,2] = den
			res[isim,3] = num/den
		} // closes loop over nsim
		vmat = quadvariance(res)
		vv = ( vmat[1,1] , vmat[2,2], vmat[3,3], vmat[1,2] )
		vvlist = vvlist \ vv

		obseai = mean(res[.,1])
			obsealist = obsealist \ obseai
		obsebi = mean(res[.,2])
			obseblist = obseblist \ obsebi
		obseci = mean(res[.,3]) // empirical var(b-hat)
			obseclist = obseclist \ obseci
	} // closes loop over ntimes

	obsea = mean(obsealist[.,1])
	obseb = mean(obseblist[.,1])
	obsec = mean(obseclist[.,1])

	obsva = mean(vvlist[.,1])
	obsvb = mean(vvlist[.,2])
	obsvc = mean(vvlist[.,3])
	obscovab = mean(vvlist[.,4])

	obappxec = obsea/obseb - obscovab/(obseb^2) + obsvb*obsea/(obseb^3) // empirical approx to var(b-hat)
	thappxec = ea/eb - covab/(eb^2) + varb*ea/(eb^3) // theoretical var(b-hat)

end

timer off 1
timer list

#delimit ;
mata: printf("{txt}\n {hline 50}\n 
No. studies: {res}%2.0f{txt}\n 
Flag: {res}%g{txt}\n 
No. sims: {res}%g×%g{txt}\n\n 
Individual components:\n 
Mean(A)\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n 
Mean(B)\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n 
Var(A)\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n 
Var(B)\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n 
Cov(AB)\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n\n 
Components of second-order expression:\n 
E(A)/E(B)\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n 
Cov(AB)/E(B)^2\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n 
Var(B)*E(A)/E(B^3)\n      Expected {res}%g{txt}\n      Observed {res}%g{txt}\n\n 
Var(A/B):\n      Empirical  {res}%g{txt}\n  
(Using second-order expression) E(A)/E(B) - Cov(AB)/E(B)^2 + Var(B)*E(A)/E(B^3)\n      Emp approx {res}%g{txt}\n      Expected   {res}%g{txt}\n\n 
Ratio of second-order expression to empirical:{txt}\n      Emp approx {res}%g{txt}\n      Expected   {res}%g{txt}\n
{hline 50}\n"
,m,flag,ntimes,nsim,ea,obsea,eb,obseb,vara,obsva,varb,obsvb,covab,obscovab,
ea/eb,obsea/obseb,covab/(eb^2),obscovab/(obseb^2),varb*ea/(eb^3),
obsvb*obsea/(obseb^3),obsec,obappxec,thappxec,obappxec/obsec,thappxec/obsec)
;
#delimit cr

log close bigsim
